using UnityEngine;

namespace MeidoPhotoStudio.Plugin;

public class DragPointPane : BasePane
{
    private enum Setting { Prop, Maid, Background, Size }

    private readonly Toggle propsCubeToggle;
    private readonly Toggle smallCubeToggle;
    private readonly Toggle maidCubeToggle;
    private readonly Toggle bgCubeToggle;

    private string header;

    public DragPointPane()
    {
        header = Translation.Get("movementCube", "header");

        propsCubeToggle = new(Translation.Get("movementCube", "props"), PropManager.CubeActive);
        propsCubeToggle.ControlEvent += (s, a) => ChangeDragPointSetting(Setting.Prop, propsCubeToggle.Value);

        smallCubeToggle = new(Translation.Get("movementCube", "small"));
        smallCubeToggle.ControlEvent += (s, a) => ChangeDragPointSetting(Setting.Size, smallCubeToggle.Value);

        maidCubeToggle = new(Translation.Get("movementCube", "maid"), MeidoDragPointManager.CubeActive);
        maidCubeToggle.ControlEvent += (s, a) => ChangeDragPointSetting(Setting.Maid, maidCubeToggle.Value);

        bgCubeToggle = new(Translation.Get("movementCube", "bg"), EnvironmentManager.CubeActive);
        bgCubeToggle.ControlEvent += (s, a) => ChangeDragPointSetting(Setting.Background, bgCubeToggle.Value);
    }

    public override void Draw()
    {
        MpsGui.Header(header);
        MpsGui.WhiteLine();

        GUILayout.BeginHorizontal();
        propsCubeToggle.Draw();
        smallCubeToggle.Draw();
        maidCubeToggle.Draw();
        bgCubeToggle.Draw();
        GUILayout.EndHorizontal();
    }

    protected override void ReloadTranslation()
    {
        header = Translation.Get("movementCube", "header");
        propsCubeToggle.Label = Translation.Get("movementCube", "props");
        smallCubeToggle.Label = Translation.Get("movementCube", "small");
        maidCubeToggle.Label = Translation.Get("movementCube", "maid");
        bgCubeToggle.Label = Translation.Get("movementCube", "bg");
    }

    private void ChangeDragPointSetting(Setting setting, bool value)
    {
        switch (setting)
        {
            case Setting.Prop:
                PropManager.CubeActive = value;
                break;
            case Setting.Background:
                EnvironmentManager.CubeActive = value;
                break;
            case Setting.Maid:
                MeidoDragPointManager.CubeActive = value;
                break;
            case Setting.Size:
                MeidoDragPointManager.CubeSmall = value;
                EnvironmentManager.CubeSmall = value;
                PropManager.CubeSmall = value;
                break;
            default:
                break;
        }
    }
}
